export interface RunnerOptions<Node, Result = void> {
    graph: [node: Node, ...rest: Node[]];
    getDeps: (node: Node) => Node[] | Promise<Node[]>;
    visit: (node: Node, signal: AbortSignal, path: Node[], depResults: DepResults<Node, Result>) => Result | Promise<Result>;
    onCycle?: (node: Node, cycle: Node[], path: Node[]) => void | Promise<void>;
    failFast?: boolean;
    signal?: AbortSignal;
}
export type DepResults<Node, Result> = Map<Node, Result | undefined>;
export interface RunnerOptionsSync<Node, Result = void> extends RunnerOptions<Node, Result> {
    getDeps: (node: Node) => Node[];
    visit: (node: Node, signal: AbortSignal, path: Node[], depResults: DepResults<Node, Result>) => Result;
    onCycle?: (node: Node, cycle: Node[], path: Node[]) => void;
}
export type SettledMap<Node, Result = void> = Map<Node, PromiseSettledResult<Result>>;
export type Callable = Function | ((...a: unknown[]) => unknown) | {
    new (...a: unknown[]): unknown;
};
export declare abstract class RunnerBase<Node, Result = void, Sync extends boolean = false, O extends Sync extends true ? RunnerOptionsSync<Node, Result> : RunnerOptions<Node, Result> = Sync extends true ? RunnerOptionsSync<Node, Result> : RunnerOptions<Node, Result>> {
    readonly results: Map<Node, Result>;
    readonly settled: SettledMap<Node, Result>;
    readonly dependents: Map<Node, Set<Node>>;
    readonly directDependents: Map<Node, Set<Node>>;
    readonly options: O;
    readonly abortController: AbortController;
    readonly failFast: boolean;
    readonly errors: unknown[];
    readonly from: Callable;
    constructor(options: O, from?: Callable);
    abstract run(): Sync extends true ? void : Promise<void>;
    abstract getDeps(n: Node): Sync extends true ? Node[] : Promise<Node[]>;
    abstract visit(n: Node, path: Node[], depResults: DepResults<Node, Result>): Sync extends true ? Result : Promise<Result>;
    abstract onCycle(n: Node, cycle: Node[], path: Node[]): Sync extends true ? void : void | Promise<void>;
    route(n: Node, d: Node): undefined | [n: Node, ...path: Node[]];
    cycleCheck(n: Node, path: Node[], d: Node): boolean;
    handleError(er: unknown, n: Node, path: Node[]): void;
    handleValue(value: Result, n: Node): void;
}
export declare class Runner<Node, Result> extends RunnerBase<Node, Result, false, RunnerOptions<Node, Result>> {
    #private;
    readonly running: Map<Node, Promise<void>>;
    getDeps(n: Node): Promise<Node[]>;
    visit(n: Node, path: Node[], depResults: DepResults<Node, Result>): Promise<Result>;
    onCycle(n: Node, cycle: Node[], path: Node[]): Promise<void>;
    run(): Promise<void>;
}
export declare class RunnerSync<Node, Result> extends RunnerBase<Node, Result, true, RunnerOptionsSync<Node, Result>> {
    #private;
    getDeps(n: Node): Node[];
    visit(n: Node, path: Node[], depResults: DepResults<Node, Result>): Result;
    onCycle(n: Node, cycle: Node[], path: Node[]): void;
    run(): Map<Node, Result>;
}
export declare const graphRun: <Node, Result>(options: RunnerOptions<Node, Result>) => Promise<Map<Node, Result>>;
export declare const graphRunSync: <Node, Result>(options: RunnerOptionsSync<Node, Result>) => Map<Node, Result>;
export declare const allSettled: <Node, Result>(options: RunnerOptions<Node, Result>) => Promise<SettledMap<Node, Result>>;
export declare const allSettledSync: <Node, Result>(options: RunnerOptionsSync<Node, Result>) => SettledMap<Node, Result>;
export declare const any: <Node, Result>(options: RunnerOptions<Node, Result>) => Promise<Result>;
export declare const anySync: <Node, Result>(options: RunnerOptionsSync<Node, Result>) => Result;
export declare const race: <Node, Result>(options: RunnerOptions<Node, Result>) => Promise<Result>;
export declare const raceSync: <Node, Result>(options: RunnerOptionsSync<Node, Result>) => Result;
//# sourceMappingURL=index.d.ts.map